#include <windows.h>
#include <stdio.h>

#define DLL32 __declspec( dllexport )


enum enumWindowCompositionAttribute
{
    WCA_ACCENT_POLICY     = 19,
    WCA_USEDARKMODECOLORS = 26
};

enum enumAccentState
{
    ACCENT_DISABLED                   = 0,
    ACCENT_ENABLE_GRADIENT            = 1,
    ACCENT_ENABLE_TRANSPARENTGRADIENT = 2,
    ACCENT_ENABLE_BLURBEHIND          = 3,
    ACCENT_ENABLE_ACRYLICBLURBEHIND   = 4,
    ACCENT_ENABLE_HOSTBACKDROP        = 5,
    ACCENT_INVALID_STATE              = 6
};

enum enumAccentFlags
{
    DRAW_NO_BORDERS    = 0x00,
    DRAW_LEFT_BORDER   = 0x20,
    DRAW_TOP_BORDER    = 0x40,
    DRAW_RIGHT_BORDER  = 0x80,
    DRAW_BOTTOM_BORDER = 0x100,
    DRAW_ALL_BORDERS   = 0x20 + 0x40 + 0x80 + 0x100
};

struct AccentPolicy
{
    enumAccentState AccentState;
    enumAccentFlags AccentFlags;
    UINT GradientColor;
    UINT AnimationId;
};

struct WINCOMPATTRDATA
{
    enumWindowCompositionAttribute Attribute;
    void* pData;
    DWORD DataSize;
};


HINSTANCE hInst;


BOOL WINAPI DllMain
(
    HINSTANCE   hInstDLL,
    DWORD       fdwReason,
    LPVOID      lpvReserved
)
{
    if( fdwReason == DLL_PROCESS_ATTACH )
        hInst = hInstDLL;
        
    return( TRUE );
}


typedef BOOL( WINAPI *SET_WINDOW_COMPOSITION_ATTRIBUTE )( HWND hWnd, WINCOMPATTRDATA* pAttrData );


DLL32 DWORD WINAPI WindowBackgroundBlur( HWND hWnd, UINT accentState, UINT accentFlags, UINT gradientColor, UINT animationId )
{
    HINSTANCE hModule;
    FARPROC procAddr;
    DWORD lastError;
    BOOL ret;
    WINCOMPATTRDATA data;
    AccentPolicy accent;
    SET_WINDOW_COMPOSITION_ATTRIBUTE SetWindowCompositionAttribute;
    char dllName[100];
    char funcName[100];

    accent.AccentState   = (enumAccentState) accentState;
    accent.AccentFlags   = (enumAccentFlags) accentFlags;
    accent.GradientColor = gradientColor;
    accent.AnimationId   = animationId;
    
    
    sprintf( dllName, "User32.dll" );
    
    hModule = LoadLibrary( dllName );
    
    if( hModule == NULL )
    {
        lastError = GetLastError();
        
        return( lastError );
    }
    
    
    sprintf( funcName, "SetWindowCompositionAttribute" );
    
    procAddr = GetProcAddress( hModule, funcName );
    
    if( procAddr == NULL )
    {
        lastError = GetLastError();
        
        return( lastError );
    }
    
    
    SetWindowCompositionAttribute = (SET_WINDOW_COMPOSITION_ATTRIBUTE) procAddr;
    
    data.Attribute = WCA_ACCENT_POLICY;
    data.pData     = (void*) &accent;
    data.DataSize  = (DWORD) sizeof( accent );
    
    ret = SetWindowCompositionAttribute( hWnd, &data );
    
    if( ret == FALSE )
    {
        lastError = GetLastError();
        
        return( lastError );
    }
    

    return( 0 );
}


DLL32 DWORD WINAPI WindowLightDarkMode( HWND hWnd, UINT lightDark )
{
    HINSTANCE hModule;
    FARPROC procAddr;
    DWORD lastError;
    BOOL ret;
    WINCOMPATTRDATA data;
    SET_WINDOW_COMPOSITION_ATTRIBUTE SetWindowCompositionAttribute;
    char dllName[100];
    char funcName[100];

    
    sprintf( dllName, "User32.dll" );
    
    hModule = LoadLibrary( dllName );
    
    if( hModule == NULL )
    {
        lastError = GetLastError();
        
        return( lastError );
    }
    
    
    sprintf( funcName, "SetWindowCompositionAttribute" );
    
    procAddr = GetProcAddress( hModule, funcName );
    
    if( procAddr == NULL )
    {
        lastError = GetLastError();
        
        return( lastError );
    }
    
    
    SetWindowCompositionAttribute = (SET_WINDOW_COMPOSITION_ATTRIBUTE) procAddr;
    
    data.Attribute = WCA_USEDARKMODECOLORS;
    data.pData     = (void*) &lightDark;
    data.DataSize  = (DWORD) sizeof( lightDark );
    
    ret = SetWindowCompositionAttribute( hWnd, &data );
    
    if( ret == FALSE )
    {
        lastError = GetLastError();
        
        return( lastError );
    }
    

    return( 0 );
}